import Flutter
import UIKit
import kameleoonClient

public class SwiftKameleoonClientFlutterPlugin: NSObject, FlutterPlugin {
    private var clients = [String : KameleoonClient]()
    private var channel : FlutterMethodChannel

    private init(channel: FlutterMethodChannel) {
        self.channel = channel
    }

  public static func register(with registrar: FlutterPluginRegistrar) {
    let channel = FlutterMethodChannel(name: "kameleoon_client_flutter", binaryMessenger: registrar.messenger())
    let instance = SwiftKameleoonClientFlutterPlugin(channel: channel)
    registrar.addMethodCallDelegate(instance, channel: channel)
  }

  public func handle(_ call: FlutterMethodCall, result: @escaping FlutterResult) {
    print(call.method)
    let arguments = call.arguments as? NSDictionary
    print(arguments)
    let siteCode = arguments?["siteCode"] as? String
    let visitorCode = arguments?["visitorCode"] as? String

    if (call.method.elementsEqual("initializeKameleoonClient")) {
        let kameleoonClient = KameleoonClientFactory.create(sitecode : siteCode!)
        clients[siteCode!] = kameleoonClient
        result("")
    }
    else if (call.method.elementsEqual("triggerExperiment")) {
        let experimentID = arguments?["experimentID"] as? Int
        do {
            try result(clients[siteCode!]?.triggerExperiment(visitorCode : visitorCode!, experimentID : experimentID!))
          } catch KameleoonError.experimentConfigurationNotFound {
            result(FlutterError.init(
                code: "ExperimentConfigurationNotFound",
                message: "Experiment configuration not found",
                details: nil
            ))
          } catch KameleoonError.sdkNotReady {
            result(FlutterError.init(
                code: "SDKNotReady",
                message: "SDK not ready",
                details: nil
            ))
          } catch KameleoonError.notTargeted {
            result(FlutterError.init(
                code: "NotTargeted",
                message: "Not targeted",
                details: nil
            ))
          } catch KameleoonError.notActivated {
            result(FlutterError.init(
                code: "NotActivated",
                message: "Not activated",
                details: nil
            ))
          } catch {
            result(FlutterError.init(
                code: "ERROR",
                message: "Error",
                details: nil
            ))
          }
    }
    else if (call.method.elementsEqual("flush")) {
        clients[siteCode!]?.flush(visitorCode : visitorCode!)
        result("")
    }
    else if (call.method.elementsEqual("trackConversion")) {
      let goalID = arguments?["goalID"] as? Int
      let revenue = arguments?["revenue"] as? Float
        clients[siteCode!]?.trackConversion(visitorCode : visitorCode!, goalId : goalID!, revenue : revenue)
        result("")
    }
    else if (call.method.elementsEqual("activateFeature")) {
      let featureKey = arguments?["featureKey"] as? String
      let featureID = arguments?["featureID"] as? Int
      do {
        let feature : Bool
        if (featureID != nil) {
            try feature = clients[siteCode!]?.activateFeature(visitorCode : visitorCode!, featureID : featureID!) == true
        }
        else {
            try feature = clients[siteCode!]?.activateFeature(visitorCode : visitorCode!, featureKey : featureKey!) == true
        }
        result(feature)
      }
      catch KameleoonError.featureConfigurationNotFound {
        result(FlutterError.init(
            code: "FeatureConfigurationNotFound",
            message: "Feature configuration not found",
            details: nil
        ))
      }
      catch KameleoonError.notTargeted {
        result(FlutterError.init(
            code: "NotTargeted",
            message: "Not targeted",
            details: nil
        ))
      }
      catch {
        result(FlutterError.init(
            code: "ERROR",
            message: "Error",
            details: nil
        ))
      }
    }
    else if (call.method.elementsEqual("obtainVisitorCode")) {
      let defaultVisitorCode = arguments?["defaultVisitorCode"] as? String
      if (defaultVisitorCode != nil) {
        result(defaultVisitorCode)
      } else {
        result(UUID().uuidString)
      }
    }
    else if (call.method.elementsEqual("obtainVariationAssociatedData")) {
      let variationID = arguments?["variationID"] as? Int
      print("variationID")
        print(variationID)
      do {
        let data : Any?
        try data = clients[siteCode!]?.obtainVariationAssociatedData(variationID : variationID!)

        let jsonData = try JSONSerialization.data(withJSONObject: data!, options: JSONSerialization.WritingOptions.prettyPrinted)
        let dataAsString = String(data: jsonData, encoding: String.Encoding.utf8)
        result(dataAsString)
      }
      catch {
        result(FlutterError.init(
            code: "ERROR",
            message: "Error",
            details: nil
        ))
      }
    }
    else if (call.method.elementsEqual("obtainFeatureVariable")) {
      let featureKey = arguments?["featureKey"] as? String
      let featureID = arguments?["featureID"] as? Int
      let variableKey = arguments?["variableKey"] as? String
        do {
            if (featureID != nil)
            {
              try result(clients[siteCode!]?.obtainFeatureVariable(featureID : featureID!, variableKey : variableKey!))
            }
            else
            {
              try result(clients[siteCode!]?.obtainFeatureVariable(featureKey : featureKey!, variableKey : variableKey!))
            }
        }
        catch {

        }
    }
    else if (call.method.elementsEqual("addData")) {
      let dataType = arguments?["dataType"] as? String
      switch dataType {
        case "CustomData":
          let id = arguments?["id"] as? Int
          let value = arguments?["value"] as? String
          clients[siteCode!]?.addData(visitorCode : visitorCode!, data : CustomData(id : id!, value : value!))
        case "PageView":
          let url = arguments?["url"] as? String
          let title = arguments?["title"] as? String
          let referrer = arguments?["referrer"] as? Int
          clients[siteCode!]?.addData(visitorCode : visitorCode!, data : PageView(url : url!, title : title!, referrer : referrer!))
        case "Interest":
          let index = arguments?["index"] as? Int
          clients[siteCode!]?.addData(visitorCode : visitorCode!, data : Interest(index : index!))
        case "Conversion":
          let goalId = arguments?["goalId"] as? Int
          let revenue = arguments?["revenue"] as? Float
          let negative = arguments?["negative"] as? Bool
          clients[siteCode!]?.addData(visitorCode : visitorCode!, data : Conversion(goalId : goalId!, revenue : revenue!, negative : negative!))
        default:
          print("Data type not compatible")
      }
      result(true);
    }
    else if (call.method.elementsEqual("runWhenReady")) {
        let timeout = arguments?["arguments"] as? Int
        clients[siteCode!]?.runWhenReady(timeoutMilliseconds : 1000) { success in
            if (success) {
                print("ready")
                self.channel.invokeMethod("setReady", arguments: true)
            }
            else {
                print("not ready")
                self.channel.invokeMethod("setReady", arguments: false)
            }
        }
    }
    result("")
  }
}
